about summary refs log tree commit diff stats
path: root/044space_surround.cc
diff options
context:
space:
mode:
authorKartik Agaram <vc@akkartik.com>2018-06-24 09:16:17 -0700
committerKartik Agaram <vc@akkartik.com>2018-06-24 09:18:20 -0700
commit23d3a02226973f80188e84fa5dcedb14413c5b7f (patch)
tree3c73284cb795e74d78e53b72df470cafca4c70cf /044space_surround.cc
parent377b00b045289a3fa8e88d4b2f129d797c687e2f (diff)
downloadmu-23d3a02226973f80188e84fa5dcedb14413c5b7f.tar.gz
4266 - space for alloc-id in heap allocations
This has taken me almost 6 weeks :(
Diffstat (limited to '044space_surround.cc')
-rw-r--r--044space_surround.cc44
1 files changed, 27 insertions, 17 deletions
diff --git a/044space_surround.cc b/044space_surround.cc
index 310672be..9957630d 100644
--- a/044space_surround.cc
+++ b/044space_surround.cc
@@ -5,27 +5,35 @@
 //: (Surrounding spaces are like lexical scopes in other languages.)
 
 :(scenario surrounding_space)
-# location 1 in space 1 refers to the space surrounding the default space, here 20.
+# location 2 in space 1 (remember that locations 0 and 1 are reserved in all
+# spaces) refers to the space surrounding the default space, here 20.
 def main [
-  # pretend address:array:location; in practice we'll use 'new'
-  10:num <- copy 5  # length
-  # pretend address:array:location; in practice we'll use 'new"
-  20:num <- copy 5  # length
+  # prepare default-space address
+  10:num/alloc-id, 11:num <- copy 0, 1000
+  # prepare default-space payload
+  1000:num <- copy 0  # alloc id of payload
+  1001:num <- copy 5  # length
+  # prepare address of chained space
+  20:num/alloc-id, 21:num <- copy 0, 2000
+  # prepare payload of chained space
+  2000:num <- copy 0  # alloc id of payload
+  2001:num <- copy 5  # length
   # actual start of this recipe
-  default-space:space <- copy 10/unsafe
+  default-space:space <- copy 10:space
   #: later layers will explain the /names: property
-  0:space/names:dummy <- copy 20/unsafe
-  1:num <- copy 32
-  1:num/space:1 <- copy 33
+  0:space/names:dummy <- copy 20:space/raw
+  2:num <- copy 94
+  2:num/space:1 <- copy 95
 ]
 def dummy [  # just for the /names: property above
 ]
-# chain space: 10 + (length) 1
-+mem: storing 20 in location 11
-# store to default space: 10 + (skip length) 1 + (index) 1
-+mem: storing 32 in location 12
-# store to chained space: (contents of location 12) 20 + (length) 1 + (index) 1
-+mem: storing 33 in location 22
+# chain space: 1000 + (alloc id) 1 + (length) 1
++mem: storing 0 in location 1002
++mem: storing 2000 in location 1003
+# store to default space: 1000 + (alloc id) 1 + (length) 1 + (index) 2
++mem: storing 94 in location 1004
+# store to chained space: (contents of location 1003) 2000 + (alloc id) 1 + (length) 1 + (index) 2
++mem: storing 95 in location 2004
 
 //: If you think of a space as a collection of variables with a common
 //: lifetime, surrounding allows managing shorter lifetimes inside a longer
@@ -33,14 +41,16 @@ def dummy [  # just for the /names: property above
 
 :(replace{} "int space_base(const reagent& x)")
 int space_base(const reagent& x) {
-  int base = current_call().default_space ? current_call().default_space : 0;
+  int base = current_call().default_space ? (current_call().default_space+/*skip alloc id*/1) : 0;
   return space_base(x, space_index(x), base);
 }
 
 int space_base(const reagent& x, int space_index, int base) {
   if (space_index == 0)
     return base;
-  return space_base(x, space_index-1, get_or_insert(Memory, base+/*skip length*/1));
+  double chained_space_address = base+/*skip length*/1+/*skip alloc id of chaining slot*/1;
+  double chained_space_base = get_or_insert(Memory, chained_space_address) + /*skip alloc id of chained space*/1;
+  return space_base(x, space_index-1, chained_space_base);
 }
 
 int space_index(const reagent& x) {